-- $Id: PICurrent.lua 98877 2009-11-18 01:19:06Z rodrigoh $

local oil = require "oil"
local oop = require "loop.base"

local log = require "openbus.util.Log"

local setmetatable = setmetatable
local tostring = tostring

---
--Objeto que permite a transferncia de informaes de um interceptador
--para o tratador de uma requisio de servio.
---
module("openbus.interceptors.PICurrent", oop.class)

---
--Constri o objeto.
--
--@return O objeto.
---
function __init(self)

  -- Os valores transferidos sero armazenados em uma tabela de chaves fracas.
  -- As chaves dessa tabela so as corotinas associadas s requisies.
  -- Assumimos que o oil cria uma nova corotina para cada requisio

  local picurrentTable = {}
  setmetatable(picurrentTable, {__mode = "k"})
  return oop.rawnew(self, {picurrentTable = picurrentTable})
end

---
--Insere um valor na tabela de transferncia
--
--@param value O valor.
---
function setValue(self, credential)
  log:interceptor("Definindo o valor {"..credential.identifier..", "..
      credential.owner.."} na "..tostring(oil.tasks.current))
  self.picurrentTable[oil.tasks.current] = credential
end

---
--Obtm um valor da tabela de transferncia
--
--@return O valor.
---
function getValue(self)
  local credential = self.picurrentTable[oil.tasks.current]
  log:interceptor("Obtendo o valor {"..credential.identifier..", "..
      credential.owner.."} da "..tostring(oil.tasks.current))
  return credential
end
