-- $Id: LeaseProvider.lua 92963 2009-06-26 02:43:09Z augusto $

local oil = require"oil"
local oop = require "loop.base"
local Timer = require "loop.thread.Timer"
local log = require "openbus.util.Log"

---
--Responsvel por verificar os leases oferecidos.
---
module ("openbus.lease.LeaseProvider", oop.class)

---
--Cria o provedor de leases.
--
--@param checkLease A funo responsvel por verificar os leases.
--@param rate O perodo de tempo entre as renovaes dos leases.
--
--@return O provedor de leases.
---
function __init(self, checkLeases, rate)
  local timer = Timer{
    scheduler = oil.tasks,
    action = checkLeases,
    rate = rate,
  }
  local newObj = oop.rawnew(self,
    { timer = timer,
    })
  newObj:startCheck()
  return newObj
end

---
--Define a funo responsvel por verificar os leases.
--
--@param checkLeases A funo responsvel por verificar os leases.
---
function setCheckLeasesFunction(self, checkLeases)
  self.timer.action = checkLeases
end

---
--DEefine o intervalo de tempo em que os leases devem ser verificados.
--
--@param rate O intervalo.
---
function setRate(self, rate)
  self.timer.rate = rate
end

---
--Inicia a execuo do verificador de leases.
---
function startCheck(self)
  self.timer:enable()
end

---
--Interrompe a execuo do verificador de leases.
---
function stopCheck(self)
  self.timer:disable()
end
